// This Javascript is responsible for the display, design and functionalities of the calendar

// set initial visibilty of events in the past
var showPastEvents = false;

// closes contact form when close-button clicked
document.addEventListener('DOMContentLoaded', function() { 
  // Attach click event to the document or a closer parent that already exists
  document.addEventListener('click', function(event) {
      // Check if the clicked element or its parent has the ID 'close-button'
      if (event.target.id === 'close-button' || event.target.closest('#close-button')) {
          var eventContactForm = document.getElementById('event-form');
          if (eventContactForm) {
              eventContactForm.style.display = 'none';
          }
        }
      });
});

// Required library
// const ICAL = require('ical.js');

// Function to fetch and process the .ics file
async function processICSFile() {
  try {
    // Fetch the .ics file
    const response = await fetch('./data/calendar.ics');
    if (!response.ok) {
      throw new Error('Failed to fetch the .ics file');
    }
    const data = await response.text();

    // Parse the .ics data
    const jcalData = ICAL.parse(data);
    const comp = new ICAL.Component(jcalData);

    // Initialize an array to hold events for FullCalendar
    let fullCalendarEvents = [];

    // Process each VEVENT component
    const vevents = comp.getAllSubcomponents('vevent');
    vevents.forEach(veventComp => {
      const vevent = new ICAL.Event(veventComp);

      // Skip all-day events
      if (vevent.component.getFirstPropertyValue('x-microsoft-cdo-alldayevent') === 'TRUE') {
        return;
      }

      // Handle recurring events
      if (vevent.isRecurring()) {
        const exdates = getExdates(vevent);
        const rrule = vevent.component.getFirstPropertyValue('rrule');
        
        // Adjust the UNTIL property to include the entire day
        if (rrule && rrule.until) {
            rrule.until = adjustUntilDate(rrule.until);
        }

        let iterator = vevent.iterator();
        let next;
        while ((next = iterator.next())) {
            let occurrenceDate = next.toJSDate();
            if (!exdates.has(occurrenceDate.toISOString().split('T')[0])) {
                fullCalendarEvents.push(formatEventForFullCalendar(vevent, occurrenceDate));
            }
        }
    } else {
        // Non-recurring events
        fullCalendarEvents.push(formatEventForFullCalendar(vevent, vevent.startDate.toJSDate()));
      }
    });

    // Filter and replace occurrences with exceptions
    return processExceptions(fullCalendarEvents);
  } catch (error) {
    console.error('Error processing .ics file:', error);
  }
}

function adjustUntilDate(until) {
  // Set the time to the end of the day to include the entire day
  until.hour = 23;
  until.minute = 59;
  until.second = 59;
  return until;
}

function getExdates(vevent) {
  let exdates = new Set();
  vevent.component.getAllProperties('exdate').forEach(exdateProp => {
    exdateProp.getValues().forEach(exdate => {
      exdates.add(exdate.toJSDate().toISOString().split('T')[0]);
    });
  });
  return exdates;
}

function formatEventForFullCalendar(vevent, occurrenceDate) {
  return {
    title: vevent.summary,
    start: occurrenceDate.toISOString(),
    end: new Date(occurrenceDate.getTime() + (vevent.endDate.toJSDate() - vevent.startDate.toJSDate())).toISOString(),
    uid: vevent.uid,
    recurrenceId: vevent.component.getFirstPropertyValue('recurrence-id')?.toJSDate().toISOString().split('T')[0]
  };
}

function processExceptions(events) {
  let eventMap = new Map();

  // First pass: Add all events to the map, exceptions will replace regular events
  events.forEach(event => {
      let date = event.recurrenceId || event.start.split('T')[0];
      let key = date + "-" + event.uid;
      eventMap.set(key, event);
  });

  // Filter the map to ensure only one occurrence per UID per date
  let finalEvents = Array.from(eventMap.values()).filter((event, _, self) => {
      let date = event.start.split('T')[0];
      let matchingEvents = self.filter(e => e.start.split('T')[0] === date && e.uid === event.uid);

      // Keep the event if it's the only one for this UID on this date, or if it's an exception
      return matchingEvents.length === 1 || event.recurrenceId;
  });

  return finalEvents;
}






// Call the function to process the .ics file
processICSFile().then(events => {
  // Here you can integrate the events with FullCalendar
  // console.log('Processed Events:', events);
  // defines toolbar based on width
  function getToolbarConfig() {
    const width = window.innerWidth;

    // Define breakpoints
    const mobileWidth = 768;

    if (width <= mobileWidth) {
      return {
        toolbar: {
          start: '',
          center: 'prev title next',
          end: 'jumpToBottom'
        },
        view: 'listYear'
      };
    } else {
      return {
        toolbar: {
          start: 'prev,next,today',
          center: 'title',
          end: 'dayGridMonth,listYear jumpToBottom'
        }
      };
    }
  }

  // Determine the initial view based on screen width
  let isMobile = window.innerWidth <= 768;
  let initialViewType = isMobile ? 'listYear' : 'listYear';
  let calendar; 

  function initializeCalendar() {
    if (!calendar) {  
      let config = getToolbarConfig();
      var calendarEl = document.getElementById('calendar');
      calendar = new FullCalendar.Calendar(calendarEl, {
        locale: 'de', // German locale
        initialView: initialViewType,
        customButtons: {
          togglePastEvents: {
              text: 'Zeige/Verstecke vergangene Termine',
              click: function() {
                  showPastEvents = !showPastEvents;
                  handlePastEventsVisibility(); 
              }
          },
          jumpToTop: {
            text: '▲',
            hint: 'Zum Kalender-Anfang',
              click: function() {
                var ef = document.getElementById('event-form');
                if(ef) {
                  document.getElementById('fc-dom-1').scrollIntoView({ behavior: 'smooth', block: 'center' });
                } else {
                  document.getElementById('form-container').scrollIntoView({ behavior: 'smooth', block: 'end' });
                }
              }
          },
          jumpToBottom: {
            text: '▼',
            hint: 'Zum Kalender-Ende',
              click: function() {
                  document.getElementById('calendar').scrollIntoView({ behavior: 'smooth', block: 'end' });
              }
          },
        },
        headerToolbar: config.toolbar,
        footerToolbar: {
          start: 'togglePastEvents',
          end: 'jumpToTop'
        },
        views: {
          dayGridMonth: { buttonText: 'Kalender'},
          listYear: { buttonText: 'Liste' },
        },
        buttonText: {
          today: 'Heute'
        },
        contentHeight: "auto",
        events: events,
        eventDisplay: 'block',

        eventContent: function(arg) {
          var start = arg.event.start;
          var end = arg.event.end;
          var title = arg.event.title;
      
          // Check if start and end are not null
          if (!start || !end) {
              console.warn('Event with missing start or end time:', arg.event);
              return { html: `<div title="${title}">${title}</div>` }; // Return a default format
          }
      
          // Format time as "[start time] - [end time] Uhr: [title]"
          var startTime = start.getHours() + ':' + (start.getMinutes() < 10 ? '0' : '') + start.getMinutes();
          var endTime = end.getHours() + ':' + (end.getMinutes() < 10 ? '0' : '') + end.getMinutes();
          var formattedTime = startTime + ' - ' + endTime + ' Uhr: ';
          
          var tooltipContent = formattedTime + title;
      
          return {
              html: `<div title="${tooltipContent}">${title}</div>`
          };
      },
      
        
        eventDidMount: function(info) {
          handlePastEventsVisibility(); 
        },   
        
        //Opens the contact form with event specific informations after clicking an event
        eventClick: function showContactForm(info) {
          let currentDate = new Date();
          let startDate = info.event.start;
          let endDate = info.event.end;

          // Check if the event's start date is in the past
          if (startDate < currentDate) {
              alert("Du kannst Dich nicht für ein Ereignis in der Vergangenheit anmelden.");
              return; // Exit the function early
          }

          // Format the date using German locale
          let formattedDate = new Intl.DateTimeFormat('de-DE', {
              year: 'numeric',
              month: 'long',
              day: '2-digit'
          }).format(startDate);

          // Format the start time using German locale
          let formattedStartTime = new Intl.DateTimeFormat('de-DE', {
              hour: '2-digit',
              minute: '2-digit',
              hour12: false  // Use 24-hour format
          }).format(startDate);

          // Format the end time using German locale
          let formattedEndTime = new Intl.DateTimeFormat('de-DE', {
              hour: '2-digit',
              minute: '2-digit',
              hour12: false  // Use 24-hour format
          }).format(endDate);
          
          function generateEventHTML (info, formattedDate, formattedStartTime, formattedEndTime) {
          return `
          <form id="event-form" class="form" style="display: none;" method="post" action="https://feedback.ruhr-uni-bochum.de/formmailer.php" ... >
          <input type="hidden" name="subject"       value="Teilnahmeanfrage von der Makerspace-Homepage" /> 
          <div class="event-subject">
              <label for="event-info" style="font-size: 18px"><strong>Deine Teilnahmeanfrage für:</strong></label>
              <input type="hidden" name="event_info" id="event-info-input" value="${info.event.title + '\nam ' + formattedDate + ' von ' + formattedStartTime + ' bis ' + formattedEndTime + ' Uhr'}" />
              <p id="event-info-display" readonly>${info.event.title + '\nam ' + formattedDate + ' von ' + formattedStartTime + ' bis ' + formattedEndTime + ' Uhr'}</p>
          </div>   
          <input type="hidden" name="recipients"    value="makerspace@ruhr-uni-bochum.de"/>
              <input type="hidden" name="mail_options" value="charset=UTF-8,Exclude=realname;honigtopf;schicken;FromAddr"/>
              <input type="hidden" name="good_url"      value="https://makerspace.ruhr-uni-bochum.de/danke/"  />       

              <div class="form-field">
                  <label for="nachname"><strong>Name:</strong></label>
                  <input name="nachname" id="nachname" type="text" maxlength="30" required/>
              </div>							

              <div class="form-field">
                  <label for="vorname"><strong>Vorname:</strong></label>
                  <input name="vorname" id="vorname" type="text" maxlength="30" required/>
              </div>							

              <div class="form-field">
                  <label for="email"><strong>E-Mail:</strong></label>
                  <input name="email" id="email" type="email" maxlength="60" required/>
              </div>			

              <div class="form-field">
                  <label for="tel"><strong>Telefonnummer:</strong></label>
                  <input name="tel" id="tel" type="tel" maxlength="40" placeholder="Optional" />
              </div>						

              <div class="form-field">
                  <label for="nachricht" id="nachricht-text"><strong>Deine Nachricht:</strong><span class='large'></span></label>
                  <textarea name="nachricht" id="nachricht" placeholder="Hier kannst Du Deine Nachricht schreiben (Limit: 2000 Zeichen)" rows="6"></textarea>
              </div> 

              <div class="honey" style="display: none;">
                  <label for="honigtopf"><strong>Please do not fill this field:</strong></label>
                  <input name="honigtopf" id="honigtopf" type="text" maxlength="50" value="" />
              </div>	

              <p style="font-size: var(--form-note-font-size);">Bitte beachte: Diese Anfrage allein berechtigt Dich noch nicht zur Teilnahme. Die Teilnahme ist erst nach Erhalt einer Bestätigungsmail von uns gültig!</p>

              <div class="form-field" style="margin-bottom: 10px; font-size: var(--form-note-font-size);">
                <label for="data-privacy" style="font-size: var(--form-note-font-size);">
                  <div style="float: left; padding: 0px;">
                    <input type="checkbox" id="data-privacy" name="data-privacy" required>
                  </div>
                  <div style="overflow: hidden; padding-left: 10px;">
                    Ich erkläre mich mit der Erhebung, Verarbeitung und Speicherung meiner hier gemachten Angaben gemäß Eurer
                    <a href="https://www.ruhr-uni-bochum.de/de/datenschutz" target="_blank" style="color: var(--link-color);">Datenschutzvereinbarung</a> einverstanden. Meine Einwilligung kann ich jederzeit durch formlose Mitteilung an Euch widerrufen.
                  </div>
                </label>
              </div>
          
              <div id=button-wrapper>
                <div class="form-field">
                    <input type="submit" id="submit-button" class="md-button" value="Absenden" name="schicken" />  
                </div>
                <div class="form-field">
                    <input type="button" id="close-button" class="md-button" value="Abbrechen" name="abbrechen" />
                </div>
              </div>				
          </form>
        </div>
      `};
      
      //scroll to form when event is clicked
      document.getElementById('form-container').innerHTML = generateEventHTML(info, formattedDate, formattedStartTime, formattedEndTime);
        let form = document.getElementById(`event-form`);
        form.style.display = 'block';
        form.scrollIntoView({ behavior: 'smooth', block: 'center' });
        
        // send message to the parent page when the form is submitted to change to thank_you-page
        document.getElementById('event-form').addEventListener('submit', function() {
          var targetOrigin = (window.location.hostname === 'localhost') ? 'http://localhost:8000' : 'https://makerspace.ruhr-uni-bochum.de';
          window.parent.postMessage('formSubmitted', targetOrigin);
        });
      
        },

        eventClassNames: function(arg) {
            if (arg.event.start < new Date()) {
              return ['past-event'];
            } else {
              return ['future-event'];
            }
          },

        allDaySlot: false, // Remove the "all-day" section from the calendar
        slotMinTime: '07:00:00', // Beginning at 7am
        slotMaxTime: '20:00:00', // Ending at 8pm
      });

      calendar.render();
    } else {
      updateToolbarConfig();
    }
  }

  // Initialize the calendar on page load
  initializeCalendar();

  // sets the toolbar and view depending on the width
  function updateToolbarConfig() {
    const config = getToolbarConfig();
    calendar.setOption('headerToolbar', config.toolbar);

  // If the configuration has the "view" key and it's set to "listYear", change the view
    if (config.view === 'listYear') {
      calendar.changeView(config.view);
    }
  }

  // adds a slight delay so that the change only occurs when the resize has been finished
  function debounce(func, wait) {
    let timeout;
    return function() {
      const context = this, args = arguments;
      clearTimeout(timeout);
      timeout = setTimeout(function() {
        func.apply(context, args);
      }, wait);
    };
  }

  // Add the resize event listener
  window.addEventListener('resize', debounce(function() {
    updateToolbarConfig();
  }, 150)); // Waits for 150ms after the last resize event to re-initialize the calendar

  
  //adjusts the height of the iframe to the height of the calendar or contact form (whatever is higher)
  function adjustHeight() {
      
    var wrapper = document.querySelector('.calendar-wrapper');

    // Initialize default height to 400
    var newHeight = 400;
    var wrapperHeight = 400;

    if (wrapper) {
      wrapperHeight = wrapper.offsetHeight;
    }

    newHeight = wrapperHeight + 10;

    // Directly set the height of the iframe in the parent
    var iframeInParent = window.parent.document.getElementById('calendar');
    if (iframeInParent) {
        iframeInParent.style.height = newHeight + 'px';
    }
  }

  // triggers adjustHeight whenever calendar-wrapper changes height
  var wrapper = document.querySelector('.calendar-wrapper');
    if (wrapper) {
      var resizeObserver = new ResizeObserver(adjustHeight);
      resizeObserver.observe(wrapper);
    }

  // Show/Hide past events
  function handlePastEventsVisibility() {
    var view = calendar.view;
    var displayStyle = view.type.includes('list') ? 'table-row' : 'block';
    var pastEvents = document.querySelectorAll('.past-event');
    pastEvents.forEach(function(eventEl) {
        eventEl.style.display = showPastEvents ? displayStyle : 'none';
    });
    if (view.type.includes('list')) {
        var pastDays = document.querySelectorAll('.fc-day-past');
        pastDays.forEach(function(dayEl) {
            dayEl.style.display = showPastEvents ? 'table-row' : 'none';
        });
    }
    var visibleEvents = calendar.getEvents();
  }
});

// Function to disable the submit button for a certain period
function debounceSubmitButton(buttonId, delay) {
    var button = document.getElementById(buttonId);
    button.disabled = true;

    // Re-enable the button after the delay
    setTimeout(function() {
        button.disabled = false;
    }, delay);
}

// Add an event listener to your form
document.addEventListener('DOMContentLoaded', function() {
    var form = document.getElementById('event-form');
    if (form) {
        form.addEventListener('submit', function(event) {
            // Debounce the submit button for 5 seconds
            debounceSubmitButton('submit-button', 5000);
        });
    }
});




